package com.ibm.scm.gui.logging;

import com.ibm.dmc.bl.logging.BLLogFactory;
import com.ibm.dmc.bl.util.BLCommonUtils;

public aspect Logger {
	pointcut annotatedMethods():
		execution (@Trace !static * com.ibm..*.*(..)) || execution (@Trace !static * pagecode..*.*(..));
	
	pointcut annotatedStaticMethods():
		execution (@Trace static * com.ibm..*.*(..)) || execution (@Trace static * pagecode..*.*(..));
	
	before() : annotatedStaticMethods()
	{
		String className = thisJoinPoint.getSignature().getDeclaringTypeName();
		String methodName = thisJoinPoint.getSignature().getName();
		Object[] arguments = thisJoinPoint.getArgs();
		
		String[] strArgs = BLCommonUtils.generateStrings4LogParams(arguments);
		BLLogFactory.getLog().trace_entry(className, methodName, strArgs);
	}
	
	before() : annotatedMethods()
	{
		String className = thisJoinPoint.getSignature().getDeclaringTypeName();
		String methodName = thisJoinPoint.getSignature().getName();
		Object[] arguments = thisJoinPoint.getArgs();
		
		//for non-static methods
		Object[] appendedArguments = new Object[arguments.length+1];
		//add this as the first param
		appendedArguments[0] = thisJoinPoint.getThis();
		System.arraycopy(arguments, 0, appendedArguments, 1, arguments.length);
		
		String[] strArgs = BLCommonUtils.generateStrings4LogParams(appendedArguments);
		BLLogFactory.getLog().trace_entry(className, methodName, strArgs);
	}

	after() returning(Object returnvalue) : annotatedMethods()
	{
		String className = thisJoinPoint.getSignature().getDeclaringTypeName();
		String methodName = thisJoinPoint.getSignature().getName();
		
		String value = BLCommonUtils.generateString4LogParam(returnvalue);
		BLLogFactory.getLog().trace_exit(className, methodName, value);
	}

	after() returning(Object returnvalue) : annotatedStaticMethods()
	{
		String className = thisJoinPoint.getSignature().getDeclaringTypeName();
		String methodName = thisJoinPoint.getSignature().getName();
		
		String value = BLCommonUtils.generateString4LogParam(returnvalue);
		BLLogFactory.getLog().trace_exit(className, methodName, value);
	}
}
